# Author: gry@ircnow.org
# Licence: public domain
# Year: 2020-2021
# Date: Jan 28th 2021 (v1)

use strict;
use warnings;

# Subclass Bot::BasicBot to provide event-handling methods.
package UppercaseBot;
use base qw(Bot::BasicBot);
use Future::Queue;
use Future::AsyncAwait;
use feature qw(signatures);

async sub send_with_ack ($self, $ack_queue, $to_send, $topic, $chan) {
   foreach my $part (@{$to_send}) {
       foreach my $msg (split '\n', $part){
           $self->say(channel => $chan, body => $msg);
       }
#       $self->say(channel => $chan, body => "Topic $topic completed. To start a next topic, write 'start TOPIC'.");
       await $self->{ack_queue}->shift;
   }
}

sub said {
    my $self      = shift;
    my $msg = shift;   # Contains the message that the bot heard.

    my @args = split ' ', $msg->{body};
    my $argslength = scalar @args;
    if($argslength eq 2){
        my $cmd = $args[0];
        my $topic = $args[1];
        my @chunks = ();
        if ($cmd eq 'start' and $topic){
            $self->{ack_queue} = Future::Queue->new;
            my $filename = "l/$topic.md";
            open(my $fh, '<:encoding(UTF-8)', $filename) or $self->say(channel => $msg->{channel}, body => "$topic - $!");
            my $break = 1; my $chunk = '';
            while (my $row = <$fh>) {
                chomp $row; if($break){$chunk = $row;$break=0;}
                elsif($row eq '----'){push (@chunks, $chunk); $break = 1;}
                else{$chunk="$chunk\n$row";}
                print "$row\n";
            }
            

            #@chunks = ("test 1\ntest 2", "3\n4\n5");
            $self->{future} = $self->send_with_ack($self->{ack_queue}, \@chunks, $topic, $msg->{channel});
            $self->{future}->on_ready(sub { delete $self->{future} });
        }
    }

    if($msg->{body} eq "next") {
        $self->{ack_queue}->push(1); # ack_queue undefined here -- can't push to undefined value
    }
#    $self->{future}->on_ready(sub { delete $self->{future} });

}



# Create an object of your Bot::BasicBot subclass and call its run method.
package main;

my $bot = UppercaseBot->new(
    server      => 'irc.anarxi.st',
    port        => '6667',
    channels    => ['#news','#news-en'],
    nick        => 'lbot',
    name        => 'bot owner: gry',
);
$bot->run();
